/**
 *
 * Base I/O device implementation.
 *
 * @file
 * Copyright &copy; Audi Electronics Venture GmbH. All rights reserved.
 *
 * $Author: A1RHEND $
 * $Date: 2013-04-11 13:09:49 +0200 (Thu, 11 Apr 2013) $
 * $Revision: 37962 $
 * @remarks
 *
 */
#ifndef _BASE_IO_DEVICE_
#define _BASE_IO_DEVICE_

namespace adtf
{

/**
 *
 * Generic I/O device event interface.
 *
 * Implementation is provided. 
 *
 * @see iodevice.cpp.
 */
class DOEXPORT IDeviceEventHandler
{
    public:
        /**
         *  Enumeration for device events.
         */
        typedef enum
        {
            ///unkown event.
            DE_Unknown      = 0,
            ///new data received.
            DE_NewData      = 1, 
            
            ///user event offset.
            DE_User         = 0x0100
        } tDeviceEventId;

    public:
        /**
         *
         * Device event handler. The OnDeviceEvent method handles incoming device events.
         *
         * @param    nEventId    [in] Event identifier (@ref tDeviceEventId).
         * @param    nParam1     [in] Event specific parameter.
         * @param    nParam2     [in] Event specific parameter.
         * @param    pvData      [in,out] Pointer to event specific data block. Can be NULL.
         * @param    nDataSize   [in] Size of event specific data block.
         * @return   Returns a standard result code.
         *
         */
        virtual tResult OnDeviceEvent(tInt nEventId, tInt nParam1, tInt nParam2, tVoid* pvData, tInt nDataSize) = 0;
};

/**
 *
 * Generic I/O device implementation.
 *
 */
class DOEXPORT cBaseIODevice : public ucom::IDevice
{
    ADTF_D(cBaseIODevice)

    protected:
        IDeviceEventHandler*    m_pEventHandler;    //!< @todo

    public:
        cBaseIODevice();
        virtual ~cBaseIODevice();

    public: // implements IDevice
        /**
         * @copydoc ucom::IDevice::Open()
         * @remarks Implements the IDevice::Open()
         */
        tResult Open(const tChar* strDeviceName, tInt nMode=0, ucom::IException** __exception_ptr=NULL);
        
        /**
         * @copydoc ucom::IDevice::Close()
         */
        tResult Close(ucom::IException** __exception_ptr=NULL);
        
        tInt Read(tVoid* pvBuffer, tInt nBufferSize);
        tInt Write(const tVoid* pvBuffer, tInt nBufferSize);
        /**
         * @copydoc ucom::IDevice::IOCtl()
         * @remarks Implements the IDevice::Open()
         */
        tInt IOCtl(tInt nCommand, tVoid* pvData=NULL, tInt nDataSize=0);

    public: // implements IObject
        tResult GetInterface(const tChar* idInterface, void ** ppvObject);
        tUInt Ref();
        tUInt Unref();
        tVoid Destroy();

    protected:
        /**
         *
         * Forward device specific event to handler. This methods is responsible to forward
         * device specific notification to the device event handler. Usually the device event
         * handler is implemented by the device filter.
         *
         * @param    nEventId     [in] Event identifier.
         * @param    nParam1      [in] Event specific parameter.
         * @param    nParam2      [in] Event specific parameter.
         * @param    pvData       [in,out] Pointer to event specific data.
         * @param    nDataSize    [in] Size of event specific data block.
         *
         * @return   Returns a standard result code.
         *
         */
        virtual tResult SetDeviceEvent(tInt nEventId,
                                       tInt nParam1=0,
                                       tInt nParam2=0,
                                       tVoid* pvData=NULL,
                                       tInt nDataSize=0);
};

} // namespace adtf

//*************************************************************************************************
#endif // _BASE_IO_DEVICE_
