/**
 *
 * Base I/O device filter implementation.
 *
 * @file
 * Copyright &copy; Audi Electronics Venture GmbH. All rights reserved.
 *
 * $Author: VG8D3AW $
 * $Date: 2014-03-31 16:30:15 +0200 (Mon, 31 Mär 2014) $
 * $Revision: 45827 $
 * @remarks
 *
 */
#include "stdafx.h"
#include <adtf_plugin_sdk.h>
using namespace ucom;

namespace adtf
{

//*************************************************************************************************
class cDeviceControl : public cKernelCyclicThread
{
    protected:
        IDeviceTrigger* m_pTarget;

    public:
        cDeviceControl(cBaseIODeviceFilter* pFilter);
        virtual ~cDeviceControl();

        tResult Create(IDeviceTrigger* pTarget);
        tResult Unref();

        tResult Run();
        tResult Stop();

        virtual tResult CyclicFunc();

    private:
        cBaseIODeviceFilter* m_pFilter;
};

//*************************************************************************************************
cDeviceControl::cDeviceControl(cBaseIODeviceFilter* pFilter) : cKernelCyclicThread()
{
    #ifdef _DEBUG
//         SetName("Device Control Tread"); // specify debug info
    #endif // _DEBUG

    m_pFilter = pFilter;
    m_pTarget = NULL;
}

//*************************************************************************************************
cDeviceControl::~cDeviceControl()
{
    Unref();
}

//*************************************************************************************************
tResult cDeviceControl::Create(IDeviceTrigger* pTarget)
{
    RETURN_IF_POINTER_NULL(pTarget);

    m_pTarget = pTarget;

    cString strID = "device";
    ucom::cObjectPtr<ucom::IObjectInfo> pFilterInfo;
    if (IS_OK(m_pFilter->GetInterface(IID_OBJECT_INFO, (tVoid**) &pFilterInfo)))
    {
        strID = pFilterInfo->OIGetInstanceName();
    }

    // create thread
    RETURN_IF_FAILED(cKernelCyclicThread::Create(cThread::TF_Suspended, NULL, NULL, 0, strID + ".devicecontrol_thread"));

    RETURN_NOERROR;
}

//*************************************************************************************************
tResult cDeviceControl::Run()
{
    cKernelCyclicThread::Run();

    RETURN_NOERROR;
}

//*************************************************************************************************
tResult cDeviceControl::Stop()
{
    if (cKernelCyclicThread::GetState() == cKernelCyclicThread::TS_Running)
    {
        cKernelCyclicThread::Suspend(tTrue);
    }

    RETURN_NOERROR;
}

//*************************************************************************************************
tResult cDeviceControl::Unref()
{
    cKernelCyclicThread::Release();

    RETURN_NOERROR;
}

//*************************************************************************************************
tResult cDeviceControl::CyclicFunc()
{
    m_pTarget->OnDeviceTrigger(NULL);

    RETURN_NOERROR;
}

/**************************************************************************/
/* Private implementation to provide object compatibility                 */
/**************************************************************************/
class cBaseIODeviceFilter::cBaseIODeviceFilterPrivate : public cADTFD<cBaseIODeviceFilter, cBaseIODeviceFilter::cBaseIODeviceFilterPrivate>
{
    friend class cBaseIODeviceFilter;
    public:
        cBaseIODeviceFilterPrivate(){};
        virtual ~cBaseIODeviceFilterPrivate(){};
};

//*************************************************************************************************
const tChar* cBaseIODeviceFilter::m_strDefaultDeviceName = "default";

//*************************************************************************************************
cBaseIODeviceFilter::cBaseIODeviceFilter(const tChar* __info) : cFilter(__info) 
{
    //d-pointer implementation not yet in use
    //ADTF_D_CREATE(cBaseIODeviceFilter);

    m_pDevice                           = NULL;
    m_pTriggerThread                    = NULL;
    m_pStatistics                       = NULL;

    // predefine properties
    SetPropertyStr("devicename",        m_strDefaultDeviceName);
    SetPropertyStr("devicename" NSSUBPROP_DESCRIPTION,
                   "Name of device to be used.");

    SetPropertyFloat("updaterate",        0);
    SetPropertyStr("updaterate" NSSUBPROP_DESCRIPTION,
                   "Limits the rate at which the device thread gets called. Use 0 to disable limitation.");

    SetPropertyBool("syncenable",       tTrue);
    SetPropertyStr("syncenable" NSSUBPROP_DESCRIPTION,
                   "If set to true each transmitted mediasample will be marked with the MSF_SyncPoint flag.");
}

//*************************************************************************************************
cBaseIODeviceFilter::~cBaseIODeviceFilter()
{
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::Init(cFilter::tInitStage eStage, ucom::IException** __exception_ptr)
{
    if (eStage == StageFirst)
    {
        m_pTriggerThread = NULL;

        m_f32UpdateRate = (tFloat32) GetPropertyFloat("updaterate");
     
        m_bSyncEnable = GetPropertyBool("SyncEnable");
        RETURN_IF_FAILED(OnCreatePins());
    }
    else if (eStage == StageNormal)
    {
        tResult nResult = ERR_NOERROR;

        cString strDeviceName = GetPropertyStr("DeviceName");
        if (strDeviceName.IsEmpty())
        {
            strDeviceName = GetPropertyStr("Source");

            if (strDeviceName.IsEmpty())
            {
                strDeviceName = m_strDefaultDeviceName;
            }
        }
        else
        {
            cString strNameLeft = strDeviceName.Left(8);
            if (cString::IsEqualNoCase(strNameLeft, "capture:"))
            {
                strDeviceName.Delete(0, 8);
            }
        }

        cString strFilterName = OIGetInstanceName();
        if (strFilterName.IsEmpty())
        {
            strFilterName = "Unnamed IO Device";
        }

        m_pStatistics = new cStatistics(strFilterName);

        if ( m_f32UpdateRate > 0)
        {
            m_nMinDelay = (tTimeStamp) (1000000 / m_f32UpdateRate);
        }
        else
        {
           m_nMinDelay = 0;
        }

        tInt nDeviceStatus;

        // ---- open device ----
        nResult = m_pDevice->Open(strDeviceName.GetPtr(), IDevice::DM_Read, __exception_ptr);
        if (IS_FAILED(nResult))
        {
            if (__exception_ptr != NULL)
            {
                __catch_exception(*__exception_ptr)
                {
                    //just forward error code 
                   RETURN_ERROR((*__exception_ptr)->GetResult());
                }
            }
            THROW_ERROR_DESC(nResult, "Could not open device");
        }

        IDevice::tDeviceInfo sDeviceInfo;
        nDeviceStatus = m_pDevice->IOCtl(IDevice::IOCTL_GetDeviceInfo, &sDeviceInfo, sizeof(sDeviceInfo));
        if (0 > nDeviceStatus)
        {
            THROW_ERROR_DESC(ERR_DEVICE_NOT_READY, "Could not query device info");
        }

        m_pDevice->IOCtl(IDevice::IOCTL_RegisterHandler, static_cast<IDeviceEventHandler*> (this));

        // ---- initialize device control thread ----
        m_pTriggerThread = new cDeviceControl(this);
        if (m_pTriggerThread == NULL)
        {
            THROW_ERROR_DESC(ERR_MEMORY, "Could not create device control thread");
        }

        nResult = ((cDeviceControl*) m_pTriggerThread)->Create(this);
        if (IS_FAILED(nResult))
        {
            THROW_ERROR_DESC(nResult, "Could not initialize device control thread");
        }

        // if read is non-blocking for device, the control
        // thread has to take care about synchronizing the calls
        tInt nCycleTime = 0;
        if ((sDeviceInfo.ui32Flags & IDevice::DF_AsyncIO) != 0)
        {
            nCycleTime = (tInt) m_nMinDelay;
        }
        THROW_IF_FAILED(((cDeviceControl*) m_pTriggerThread)->SetCycleTime(nCycleTime));
    }

    return cFilter::Init(eStage, __exception_ptr);
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::Start(ucom::IException** __exception_ptr)
{
    m_nLastDataTime = -1;

    m_pStatistics->Reset();

    if (m_pTriggerThread != NULL)
    {
        ((cDeviceControl*) m_pTriggerThread)->Run();
    }

    return cFilter::Start(__exception_ptr);
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::Stop(ucom::IException** __exception_ptr)
{
    if (m_pTriggerThread != NULL)
    {
        ((cDeviceControl*) m_pTriggerThread)->Stop();
    }

    return cFilter::Stop(__exception_ptr);
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::Shutdown(cFilter::tInitStage eStage, ucom::IException** __exception_ptr)
{
    if (m_pTriggerThread != NULL)
    {
        ((cDeviceControl*) m_pTriggerThread)->Release();
        delete (cDeviceControl*) m_pTriggerThread;
        m_pTriggerThread = NULL;
    }

    if (m_pDevice != NULL)
    {
        m_pDevice->IOCtl(IDevice::IOCTL_UnregisterHandler, static_cast<IDeviceEventHandler*> (this));
        m_pDevice->Close();
    }

    if (m_pStatistics != NULL)
    {
        delete m_pStatistics;
        m_pStatistics = NULL;
    }

    cFilter::Shutdown(eStage, __exception_ptr);

    RETURN_NOERROR;
}

tResult cBaseIODeviceFilter::OnDeviceTrigger(tVoid* pvUserData)
{
    return ReadAndTransmitData();
}

tResult cBaseIODeviceFilter::CheckUpdateRate()
{
    if (NULL == _clock)
    {
        RETURN_NOERROR;
    }

    tTimeStamp tmTimeStamp = _clock->GetStreamTime();

    if (m_nMinDelay != 0)
    {
        if (m_nLastDataTime != -1 &&
            (tmTimeStamp - m_nLastDataTime) < m_nMinDelay)
        {
            // drop data due to frame rate limitation.
            RETURN_ERROR(ERR_CANCELLED);
        }

        m_nLastDataTime += m_nMinDelay;

        if ((tmTimeStamp - m_nLastDataTime) > m_nMinDelay)
        {
            m_nLastDataTime = tmTimeStamp - m_nMinDelay;
        }
        else if (m_nLastDataTime > tmTimeStamp)
        {
            m_nLastDataTime = tmTimeStamp;
        }
    }

    RETURN_NOERROR;
}

tResult cBaseIODeviceFilter::ReadAndTransmitData()
{
    if (IS_FAILED(CheckUpdateRate()))
    {
        RETURN_NOERROR;
    }

    tTimeStamp tmTimeStamp = 0;
    
    if (NULL != _clock)
    {
        tmTimeStamp = _clock->GetStreamTime();
    }

    tInt nDeviceStatus = m_pDevice->IOCtl(IDevice::IOCTL_GetDataSize);
    if (0 > nDeviceStatus)
    {
        LOG_ERROR("device not ready");
        RETURN_ERROR(ERR_DEVICE_NOT_READY);
    }

    if (0 == nDeviceStatus)
    {
        RETURN_NOERROR;
    }

    tInt nBufferSize = nDeviceStatus;

    cObjectPtr<IMediaSample> pNewSample;
    RETURN_IF_FAILED(_runtime->CreateInstance(OID_ADTF_MEDIA_SAMPLE, IID_ADTF_MEDIA_SAMPLE, (tVoid**) &pNewSample));

    RETURN_IF_FAILED(pNewSample->AllocBuffer(nBufferSize));

    tVoid* pBuffer;
    RETURN_IF_FAILED(pNewSample->WriteLock(&pBuffer));

    nDeviceStatus = m_pDevice->Read(pBuffer, nBufferSize);

    pNewSample->Unlock(pBuffer);

    if (0 == nDeviceStatus)
    {
        // no data
        RETURN_NOERROR;
    }

    if (-1 == nDeviceStatus)
    {
        LOG_ERROR("could not read from device");
        RETURN_ERROR(ERR_DEVICE_IO);
    }

    tTimeStamp tmSample;
    nDeviceStatus = m_pDevice->IOCtl(IDevice::IOCTL_GetDataTime, &tmSample, sizeof(tmSample));
    if (0 > nDeviceStatus)
    {
        tmSample = tmTimeStamp;
    }
    else
    {
        tmSample = Sync2RefTime(tmSample);
    }

    pNewSample->SetTime(tmSample);
    pNewSample->SetFlags(m_bSyncEnable ? IMediaSample::MSF_SyncPoint : IMediaSample::MSF_None);

    OnTransmitData(pNewSample);

    m_pStatistics->Update();

    RETURN_NOERROR;
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::OnCreatePins()
{
    RETURN_ERROR(ERR_NOT_IMPL);
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::OnTransmitData(IMediaSample* pSample)
{
    RETURN_ERROR(ERR_NOT_IMPL);
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::OnGetMediaType(IMediaType** ppMediaType)
{
    RETURN_ERROR(ERR_NOT_IMPL);
}

//*************************************************************************************************
tResult cBaseIODeviceFilter::OnDeviceEvent(tInt nEventId, tInt nParam1, tInt nParam2, tVoid* pvData, tInt nDataSize)
{
    if (nEventId == IDeviceEventHandler::DE_NewData)
    {
        ReadAndTransmitData();
        RETURN_NOERROR;
    }

    RETURN_ERROR(ERR_NOT_IMPL);
}

} // namespace adtf
