/**
 *
 * Base I/O device filter implementation.
 *
 * @file
 * Copyright &copy; Audi Electronics Venture GmbH. All rights reserved.
 *
 * $Author: wartand $
 * $Date: 2014-03-12 09:00:13 +0100 (Mit, 12 Mär 2014) $
 * $Revision: 45069 $
 *
 * @remarks
 *
 */
#ifndef _ADTF_BASE_IO_DEVICE_FILTER_HEADER_
#define _ADTF_BASE_IO_DEVICE_FILTER_HEADER_

namespace adtf
{

/**
 *
 * I/O device trigger.
 *
 */
class DOEXPORT IDeviceTrigger
{
    public:
        /**
         *
         * Cyclic device trigger. The OnDeviceTrigger method is periodically called by the device
         * control thread depending on the device operation flags.
         *
         * @param    pvUserData  [in,out] Currently not used.
         * @return   Returns a standard result code.
         *
         */
        virtual tResult OnDeviceTrigger(tVoid* pvUserData) = 0;
};

/**
 * Base generic I/O device filters.
 * 
 * It defines the basic implementation that is needed to use a @ref cBaseIODevice.
 *
 * \n For implementing the cMyDevice shown in this example use the @ref adtf::cBaseIODevice.
 * 
 *
 * \n To use this default filter declare:
 * \code 
 * cMyDeviceFilter : public adtf::cBaseIODeviceFilter
 * {
 *    ADTF_FILTER("adtf.io.my_dev_filter", "My Device Filter", OBJCAT_SensorDevice);
 *    private:
 *          cMyDeviceFilter m_oMyDevice;
 *    public:
 *          cMyDeviceFilter(const tChar*);
 *          virtual ~cMyDeviceFilter();
 * };
 * \endcode 
 *
 * \n The implementation is very easy like:
 * \code 
 * ADTF_FILTER_PLUGIN("My Device Filter Plugin", "adtf.io.my_dev_filter", cMyDeviceFilter)
 *
 * cMyDeviceFilter::cMyDeviceFilter(const tChar* __info) 
 *                   : adtf::cBaseIODeviceFilter(__info) 
 * {
 *      m_pDevice = &m_oMyDevice;
 * }
 * 
 * cMyDeviceFilter::~cMyDeviceFilter() 
 * {
 * }
 * \endcode 
 * @see adtf::cBaseIODevice, iodevice.cpp, iodevicefilter.cpp.
 * @see The examples added within the SDK will show the using of this (empty_can) and the implementation of the base classes are added.
 * 
 */
class DOEXPORT cBaseIODeviceFilter : public adtf::cFilter,
                                     public IDeviceTrigger,
                                     public IDeviceEventHandler
{
    ADTF_D(cBaseIODeviceFilter)

    protected:
        static const tChar*                     m_strDefaultDeviceName;     //!< @todo
        
    protected:
        adtf_util::cStatistics*                 m_pStatistics;              //!< @todo
        ucom::IDevice*                          m_pDevice;                  //!< @todo
        tHandle                                 m_hStorageHandle;           //!< @todo
        tTimeStamp                              m_nLastDataTime;            //!< @todo
        cKernelThread*                          m_pTriggerThread;           //!< @todo
        tFloat32                                m_f32UpdateRate;            
        //!< specifies the update rate in Hz 
        tTimeStamp                              m_nMinDelay;                //!< @todo
        tBool                                   m_bSyncEnable;              //!< @todo
        
    public:
        /**
         * Constructor
         */
        cBaseIODeviceFilter(const tChar*);
        virtual ~cBaseIODeviceFilter();

    public: // overrides cFilter
        /**
         * @copydoc cFilter::Init
         * @remarks overrides @ref cFilter::Init
         */
        tResult Init(cFilter::tInitStage eStage, ucom::IException** __exception_ptr=NULL);
        
        /**
         * @copydoc cFilter::Start
         * @remarks overrides @ref cFilter::Start
         */
        tResult Start(ucom::IException** __exception_ptr=NULL);
        
        /**
         * @copydoc cFilter::Stop
         * @remarks overrides @ref cFilter::Stop
         */
        tResult Stop(ucom::IException** __exception_ptr=NULL);
        
        /**
         * @copydoc cFilter::Shutdown
         * @remarks overrides @ref cFilter::Shutdown
         */
        tResult Shutdown(cFilter::tInitStage eStage, ucom::IException** __exception_ptr=NULL);

    public: // implements IDeviceTrigger
        /**
         * @copydoc IDeviceTrigger::OnDeviceTrigger
         * @remarks overrides @ref IDeviceTrigger::OnDeviceTrigger
         */
        tResult OnDeviceTrigger(tVoid* pvUserData);

    public: // implements IDeviceEventHandler
        /**
         * @copydoc IDeviceEventHandler::OnDeviceEvent
         * @remarks overrides @ref IDeviceEventHandler::OnDeviceEvent
         */
        tResult OnDeviceEvent(tInt nEventId, tInt nParam1, tInt nParam2, tVoid* pvData, tInt nDataSize);

    protected: // base methods
        /**
         *
         * Reads and transmits data. This methods reads data from the device, allocs a media sample
         * and finally transmits all data.
         *
         * @retval ERR_NOERROR Transmission is success or no new data available
         * @retval ERR_DEVICE_NOT_READY Device is not ready
         * @retval ERR_DEVICE_IO cannot read from device
         * @return other return codes results in problems with media samples
         *
         */
        virtual tResult ReadAndTransmitData();

        /**
         *
         * Create and initialize all pin instances. This methods is responsible for creating and
         * initializing all pin instances. For that purpose, the method might also declare the
         * input/output media type descriptors.
         *
         * @return   Returns a standard result code.
         *
         */
        virtual tResult OnCreatePins();

        /**
         *
         * Transmit data. Overwrite this method to map output data to the corresponding output pins
         * and to actually transmit a sample.
         *
         * @param [in] pSample sample to be transmitted
         * @return   Returns a standard result code.
         *
         */
        virtual tResult OnTransmitData(IMediaSample* pSample);

        /**
         *
         * Retrieve type information. Overwrite this method to offer type info to the caller.
         * A standard implementation might forward the call to the output pin's GetMediaType()
         * method.
         *
         * @param [in] ppMediaType pointer to a media type instance
         * @return   Returns a standard result code.
         *
         */
        virtual tResult OnGetMediaType(IMediaType** ppMediaType);

        /**
         *
         * Check update rate limit. Overwrite this method to do manual update rate control.
         *
         * @return   Returns no error if update is needed and error-cancelled for update rate
         *           limitation.
         *
         */
        virtual tResult CheckUpdateRate();

};

} // namespace adtf

//*************************************************************************************************
#endif // _ADTF_BASE_IO_DEVICE_FILTER_HEADER_
