/**
*
* Class for COM port communication.
*
* @file
 * Copyright &copy; Audi Electronics Venture GmbH. All rights reserved
*
 * $Author: ELAMIHA $
 * $Date: 2014-05-07 19:06:16 +0200 (Wed, 07 May 2014) $
 * $Revision: 633 $
*
* @remarks
*
*/
#ifndef _UTILS_SERIALDEVICE_HEADER
#define _UTILS_SERIALDEVICE_HEADER

namespace A_UTILS_NS
{

/**
 * Class for accessing serial devices.
 */
class cSerialDevice /* : public cCharacterDevice */
{
    A_UTILS_D(cSerialDevice)

    protected:
#ifdef WIN32
        tHandle     m_hDevice;  //!< Device handle
#else
        tInt        m_nDev;     //!< Device handle
#endif

    public:
        enum
        {
            SER_EVENPARITY,
            SER_MARKPARITY,
            SER_NOPARITY,
            SER_ODDPARITY,
            SER_SPACEPARITY
        };

        enum
        {
            SER_ONESTOPBIT,
            SER_ONE5STOPBITS,
            SER_TWOSTOPBITS
        };

    public:
        /**
         * Default constructor.
         */
        cSerialDevice();

        /**
         * Destructor.
         */
        virtual ~cSerialDevice();

        /**
         * Opens a serial device
         * @param strDeviceName [in] The device name.
         * @param nBaudRate     [in] The baud rate.
         * @param nParity       [in] The parity.
         * @param nDataBits     [in] The amount of data bits.
         * @param nStopBits     [in] The amount of stop bits.
         * @param nRxQueue      [in] The receive queue size.
         * @param nTxQueue      [in] The transmission queue size.
         * @param nReadTimeout  [in] The timeout for read operations in microseconds.
         * @return Standard result.
         * @remarks only win32: the strDeviceName "COM1" through "COM9" can be used. A general way to specify the COM-ports (inclusive the numbering greather than 9) is the "\\\\.\\COMx". Further information can be found at http://support.microsoft.com/kb/115831/en-us
         */
        tResult Open(const tChar* strDeviceName, tInt nBaudRate=9600, tInt nParity=SER_NOPARITY, tInt nDataBits=8, tInt nStopBits=SER_ONESTOPBIT, tInt nRxQueue=4096, tInt nTxQueue=4096, tTimeStamp nReadTimeout=0);

        /**
         * Closes a serial device
         * @return Standard result.
         */
        tResult Close();

        /**
         * Read data from the serial device.
         * @param pBuffer [in] The buffer to fill.
         * @param nBufferSize [in] How many bytes should be read.
         * @return The actual amount of bytes read.
         */
        tInt32 Read(tVoid* pBuffer, tInt32 nBufferSize);

        /**
         * Write data to the serial device.
         * @param pBuffer [in] The data to write.
         * @param nBufferSize [in] How many bytes should be written.
         * @return The actual amount of bytes written.
         */
        tInt32 Write(const tVoid* pBuffer, tInt32 nBufferSize);

        /**
         * Writes a string to the device by writing it byte after byte.
         * @param strCmd [in] The string to write
         * @return Standard result.
         */
        tResult WriteByteByByte(const cString &strCmd);

        /**
         * Checks if the serial device is opened.
         * @return tTrue if the serial device is opened, otherwise tFalse.
         * @rtsafe
         */
        tBool IsConnected();

    public:
        /**
         * Checks if the specified device is avaliable.
         * @param strDeviceName [in] The device name.
         * @return Standard result.
         */
        static tResult CheckForDevice(const tChar* strDeviceName);
        /**
         * Returns a bitmask of all avaliable serial devices.
         * @return A bitmask of all avaliable serial devices.
         */
        static tUInt32 GetDeviceMask();

        /**
         * Returns a bitmask of all unavaliable serial devices.
         * @return A bitmask of all unavaliable serial devices.
         */
        static tUInt32 GetUnusedDeviceMask();
        /**
         * Returns the name of the first unused serial device.
         * @param strDeviceName [out] The name of the first unused serial device.
         * @return Standard result.
         */
        static tResult FindUnusedDevice(cString& strDeviceName);
};

} //namespace A_UTILS_NS

#endif //_UTILS_SERIALDEVICE_HEADER
